/*
  This code is released under the Antivirial License.

  http://www.ecstaticlyrics.com/antiviral/

  The only condition of the license is that you may not change the license
  terms.  Because of this, you may *not* include this code in GPL-licensed
  projects, as the GPL requires that the entire project be GPL-licensed.
*/

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <unistd.h>
#include <fcntl.h>

#include "easy.h"
#include "thread.h"
#include "error.h"
#include "option.h"

#include "network.h"

#ifdef WINDOWS
  #define error_code WSAGetLastError()
  #define ECONNREFUSED WSAECONNREFUSED
  #define EHOSTUNREACH WSAEHOSTUNREACH
  #define ENETUNREACH WSAENETUNREACH
  #define ETIMEDOUT WSAETIMEDOUT
  #define EINVAL WSAEINVAL
  #define ECONNRESET WSAECONNRESET
  #define ECONNABORTED WSAECONNABORTED
  static HANDLE mutex;
#else
  #define error_code errno
  #define INVALID_SOCKET -1
  #define SOCKET_ERROR -1
  #define closesocket close
  static pthread_mutex_t mutex;
#endif

static volatile int thread_run_flag = 0;
static volatile int thread_exit_flag = 0;
static volatile int initialized = 0;

static void *thread (void *argument) {

  int success = 0;

  int return_value;

  struct addrinfo hints = {};
  hints.ai_family = AF_UNSPEC;
  hints.ai_socktype = SOCK_STREAM;
  struct addrinfo *results = NULL;

  fprintf(stderr, "Finding address for %s...\n", option_rtl_tcp_address);
  return_value = getaddrinfo(option_rtl_tcp_address, option_rtl_tcp_port, &hints, &results);

  if (return_value != 0) {
    if (return_value == EAI_NONAME) {
      fprintf(stderr, "getaddrinfo(): Unable to resolve server address.\n");
    } else {
      fprintf(stderr, "getaddrinfo(): Unknown error %s.\n", gai_strerror(return_value));
    };
  } else {

    int count = 0;
    for (struct addrinfo *p = results; p != NULL; p = p->ai_next) count++;
    fprintf(stderr, "...getaddrinfo() returned %d result%s.\n", count, count == 1 ? "" : "s");

    struct addrinfo *result = results;
    for (int index = 1; result != NULL; result = result->ai_next) {
      fprintf(stderr, "Connection attempt %d: ", index++);

      return_value = socket(result->ai_family, result->ai_socktype, result->ai_protocol);

      if (return_value == INVALID_SOCKET) {
        fprintf(stderr, "socket(): %s\n", error_string(error_code));
        continue;
      };

      network_socket = return_value;

      return_value = connect(network_socket, result->ai_addr, result->ai_addrlen);

      if (return_value == SOCKET_ERROR) {
        if (error_code == ECONNREFUSED) {
          fprintf(stderr, "The server is not accepting connections.\n");
        } else if (error_code == EHOSTUNREACH) {
          fprintf(stderr, "Unable to contact the server.\n");
        } else if (error_code == ENETUNREACH) {
          fprintf(stderr, "Unable to access the network.\n");
        } else if (error_code == ETIMEDOUT) {
          fprintf(stderr, "Received no response from the server.\n");
        } else if (error_code == EINVAL) {
          fprintf(stderr, "The connection failed for unknown reasons.\n");
        } else {
          fprintf(stderr, "Unknown error %d received from connect().\n", error_code);
        };
        continue;
      };

      #ifdef UNIX
        int flags = fcntl(network_socket, F_GETFL, 0);
        fcntl(network_socket, F_SETFL, flags | O_NONBLOCK);
      #endif
      #ifdef WINDOWS
        int mode = 1;
        ioctlsocket(network_socket, FIONBIO, (void *) &mode);
      #endif

      success = 1;
      break;
    };

  };

  if (!success) {
    closesocket(network_socket);
    fprintf(stderr, "Failed to connect to rtl_tcp at %s port %s\n", option_rtl_tcp_address, option_rtl_tcp_port);
    exit(1);
  };

  fprintf(stderr, "Connection to rtl_tcp successfully established.\n");

  initialized = 1;

  thread_run_flag = 0;

};

void network_initialize () {

/*
  thread_run_flag = 1;
  thread_exit_flag = 0;
  thread_create(thread, NULL);

  while (!initialized) easy_sleep(0.001);
*/

  thread(NULL);

};

void network_terminate () {

/*
  if (thread_run_flag) {
    thread_exit_flag = 1;
    barrier_write();
    while (thread_run_flag) {
      easy_sleep(0.01);
    };
  };
*/

};
