/*
  This code is released under the Antivirial License.

  http://www.ecstaticlyrics.com/antiviral/

  The only condition of the license is that you may not change the license
  terms.  Because of this, you may *not* include this code in GPL-licensed
  projects, as the GPL requires that the entire project be GPL-licensed.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <alsa/asoundlib.h>

// shit for stat()...
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "easy.h"
#include "memory.h"

char *option_rtl_tcp_address = "127.0.0.1";
char *option_rtl_tcp_port = "1234";
int option_upconverter_frequency = 125000000;
int option_waterfall_slow = 8;
int option_waterfall_smooth = 4;
//int option_filter_quality = 0;
int option_less_stuff = 0;
int option_my_gpu_sucks = 0;
char *option_alsa_pcm_device = "default";

void option_parse (int argc, char **argv) {

  for (int i = 1; i < argc; i++) {
    if (strcmp(argv[i], "--help") == 0) {
      HELP:;
      extern char help_txt[];
      extern int help_txt_size;
      fprintf(stderr, "\n");
      fwrite(help_txt, help_txt_size, 1, stderr);
      fprintf(stderr, "\n");
      exit(1);
    } else if (strcmp(argv[i], "--server") == 0) {
      int ok = 0;
      if (i + 1 < argc && *argv[i + 1] != 0) {
        option_rtl_tcp_address = argv[i + 1];
        ok = 1;
        i++;
      };
      if (!ok) {
        fprintf(stderr, "\nOption --server requires a server address.\n");
        fprintf(stderr, "Use --help if you need more help.\n\n");
        exit(1);
      };
    } else if (strcmp(argv[i], "--port") == 0) {
      int ok = 0;
      if (i + 1 < argc && *argv[i + 1] != 0) {
        option_rtl_tcp_port = argv[i + 1];
        ok = 1;
        i++;
      };
      if (!ok) {
        fprintf(stderr, "\nOption --port requires a port address.\n");
        fprintf(stderr, "Use --help if you need more help.\n\n");
        exit(1);
      };
    } else if (strcmp(argv[i], "--offset") == 0) {
      int ok = 0;
      if (i + 1 < argc && *argv[i + 1] != 0) {
        char *end;
        int t = strtol(argv[i + 1], &end, 0);
        if (*end == 0) {
          if (t > -1000000 && t < 1000000) t *= 1000000;
          option_upconverter_frequency = t;
          ok = 1;
        };
        i++;
      };
      if (!ok) {
        fprintf(stderr, "\nOption --offset requires an integer frequency value in Hz.\n");
        fprintf(stderr, "Use --help if you need more help.\n\n");
        exit(1);
      };
    } else if (strcmp(argv[i], "--slow") == 0) {
      int ok = 0;
      if (i + 1 < argc && *argv[i + 1] != 0) {
        char *end;
        int t = strtol(argv[i + 1], &end, 0);
        if (*end == 0) {
          if (t >= 4 && t <= 10) {
            option_waterfall_slow = t;
            ok = 1;
          };
        };
        i++;
      };
      if (!ok) {
        fprintf(stderr, "\nOption --slow requires an integer in the range from 4 to 10.\n");
        fprintf(stderr, "Use --help if you need more help.\n\n");
        exit(1);
      };
    } else if (strcmp(argv[i], "--smooth") == 0) {
      int ok = 0;
      if (i + 1 < argc && *argv[i + 1] != 0) {
        char *end;
        int t = strtol(argv[i + 1], &end, 0);
        if (*end == 0) {
          if (t >= 0 && t <= 10) {
            option_waterfall_smooth = t;
            ok = 1;
          };
        };
        i++;
      };
      if (!ok) {
        fprintf(stderr, "\nOption --smooth requires an integer in the range from 0 to 10.\n");
        fprintf(stderr, "Use --help if you need more help.\n\n");
        exit(1);
      };
/*
    } else if (strcmp(argv[i], "--quality") == 0) {
      int ok = 0;
      if (i + 1 < argc && *argv[i + 1] != 0) {
        char *end;
        int t = strtol(argv[i + 1], &end, 0);
        if (*end == 0) {
          if (t >= 0 && t <= 2) {
            option_filter_quality = t;
            ok = 1;
          };
        };
        i++;
      };
      if (!ok) {
        fprintf(stderr, "\nOption --quality requires an integer in the range from 0 to 2.\n");
        fprintf(stderr, "Use --help if you need more help.\n\n");
        exit(1);
      };
*/
    } else if (strcmp(argv[i], "--less") == 0) {
      option_less_stuff = 1;
    } else if (strcmp(argv[i], "--lamegpu") == 0) {
      option_my_gpu_sucks = 1;
    } else if (strcmp(argv[i], "--device") == 0 && i + 1 >= argc) {
      LIST:
      fprintf(stdout, "\nAvailable PCM devices:\n\n");
      {
        char **hints;
        int error = snd_device_name_hint(-1, "pcm", (void***) &hints);
        if (error < 0) easy_two_fucks("snd_device_name_hint()", snd_strerror(error));
        char** n = hints;
        while (*n != NULL) {
          char *name = snd_device_name_get_hint(*n, "NAME");
          if (name != NULL && 0 != strcmp("null", name)) {
            fprintf(stdout, "  %s\n", name);
            free(name);
          };
          n++;
        };
        snd_device_name_free_hint((void**) hints);
      };
      fprintf(stdout, "\nExample:  ./natpos --device plughw:0,0\n");
      fprintf(stdout, "If you just want to use your other sound card, try using \"plughw:1,0\"\n\n");
      exit(0);
    } else if (strcmp(argv[i], "--device") == 0) {
      if (i + 1 < argc) {
        option_alsa_pcm_device = argv[i + 1];
        i++;
      } else {
        goto LIST;
        fprintf(stderr, "\nOption --device requires a ALSA device name.\n");
        fprintf(stderr, "Use --help if you need more help.\n");
        exit(1);
      };
    } else {
      fprintf(stderr, "\nUnrecognized option '%s'\n", argv[i]);
      fprintf(stderr, "Use --help if you need more help.\n\n");
      exit(1);
    };
  };

  // Turns out the value for "--smooth" doesn't work like I thought, so
  // I guess I'll make it work like I thought it worked, as that's easier
  // to understand than how it actually works.

  option_waterfall_smooth = option_waterfall_slow - option_waterfall_smooth;
  if (option_waterfall_smooth < 0) option_waterfall_smooth = 0;

  // The two extra waterfalls seem to be too much for the Raspberry Pi,
  // causing it to completely lock up, which is annoying as all fuck.
  // Since the internet offers no good ideas for how to detect whether we
  // are running on a Raspberry Pi, I guess we'll just do this instead:

  struct stat whatever;
  if (0 == stat("/usr/bin/raspi-config", &whatever)) {
    if (!option_less_stuff) {
      fprintf(stderr, "Using --less anyway, as you seem to be on a Raspberry Pi.\n");
      option_less_stuff = 1;
    };
  };

};
